<?php

namespace Drupal\eca_push_framework\Plugin\ECA\Event;

use Drupal\eca\Plugin\DataType\DataTransferObject;
use Drupal\eca\Plugin\ECA\Event\EventBase;
use Drupal\push_framework\ChannelEvents;
use Drupal\push_framework\Event\ChannelEventBase;
use Drupal\push_framework\Event\ChannelEventLanguageBase;
use Drupal\push_framework\Event\ChannelPostRender;
use Drupal\push_framework\Event\ChannelPreBuild;
use Drupal\push_framework\Event\ChannelPrepareTemplates;
use Drupal\push_framework\Event\ChannelPreRender;

/**
 * Plugin implementation of the ECA Events for the Push Framework.
 *
 * @EcaEvent(
 *   id = "push_framework_channel",
 *   deriver = "Drupal\eca_push_framework\Plugin\ECA\Event\ChannelEventDeriver"
 * )
 */
class ChannelEvent extends EventBase {

  /**
   * {@inheritdoc}
   */
  public static function definitions(): array {
    $events = [];
    $events['prepare_templates'] = [
      'label' => 'Prepare templates',
      'event_name' => ChannelEvents::PREPARE_TEMPLATES,
      'event_class' => ChannelPrepareTemplates::class,
    ];
    $events['pre_build'] = [
      'label' => 'Pre build notification',
      'event_name' => ChannelEvents::PRE_BUILD,
      'event_class' => ChannelPreBuild::class,
    ];
    $events['pre_render'] = [
      'label' => 'Pre render notification',
      'event_name' => ChannelEvents::PRE_RENDER,
      'event_class' => ChannelPreRender::class,
    ];
    $events['post_render'] = [
      'label' => 'Post render notification',
      'event_name' => ChannelEvents::POST_RENDER,
      'event_class' => ChannelPostRender::class,
    ];
    return $events;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function getData(string $key): mixed {
    $event = $this->getEvent();

    if ($key === 'push_notification' && $event instanceof ChannelEventBase) {
      $data = DataTransferObject::create();
      $data->set('channel',
        DataTransferObject::create($event->getChannelPlugin()->getPluginId()));
      $data->set('recipient', DataTransferObject::create($event->getUser()));
      $data->set('entity', DataTransferObject::create($event->getEntity()));
      $data->set('display_mode', DataTransferObject::create($event->getDisplayMode()));
      if ($event instanceof ChannelPrepareTemplates) {
        $data->set('subject', DataTransferObject::create($event->getSubject()));
        $data->set('body', DataTransferObject::create($event->getBody()));
        $data->set('text_format', DataTransferObject::create($event->getTextFormat()));
        $data->set('is_html', DataTransferObject::create($event->isHtml()));
      }
      if ($event instanceof ChannelEventLanguageBase) {
        $data->set('language_key', DataTransferObject::create($event->getLanguageKey()));
      }
      if ($event instanceof ChannelPreRender) {
        $data->set('elements', DataTransferObject::create($event->getElements()));
      }
      if ($event instanceof ChannelPostRender) {
        $data->set('output', DataTransferObject::create($event->getOutput()));
      }
      return $data;
    }

    return parent::getData($key);
  }

}
